@@ -12,6 +12,7 @@ module RSpec
 
       def initialize(configuration=RSpec.configuration)
         @configuration = configuration
+        configuration.world = self
         @example_groups = []
         @example_group_counts_by_spec_file = Hash.new(0)
         @filtered_examples = Hash.new do |hash, group|
@@ -47,12 +48,10 @@ module RSpec
 
       # @api private
       #
-      # Register an example group.
-      def register(example_group)
+      # Records an example group.
+      def record(example_group)
         @configuration.on_example_group_definition_callbacks.each { |block| block.call(example_group) }
-        example_groups << example_group
         @example_group_counts_by_spec_file[example_group.metadata[:absolute_file_path]] += 1
-        example_group
       end
 
       # @private
@@ -93,6 +92,19 @@ module RSpec
         FlatMap.flat_map(all_example_groups) { |g| g.examples }
       end
 
+      # @private
+      # Traverses the tree of each top level group.
+      # For each it yields the group, then the children, recursively.
+      # Halts the traversal of a branch of the tree as soon as the passed block returns true.
+      # Note that siblings groups and their sub-trees will continue to be explored.
+      # This is intended to make it easy to find the top-most group that satisfies some
+      # condition.
+      def traverse_example_group_trees_until(&block)
+        example_groups.each do |group|
+          group.traverse_tree_until(&block)
+        end
+      end
+
       # @api private
       #
       # Find line number of previous declaration.
@@ -208,6 +220,27 @@ module RSpec
           1 # exit code
         )
       end
+
+      # @private
+      # Provides a null implementation for initial use by configuration.
+      module Null
+        def self.registered_example_group_files
+          []
+        end
+
+        def self.traverse_example_group_trees_until
+        end
+
+        # :nocov:
+        def self.example_groups
+          []
+        end
+
+        def self.all_example_groups
+          []
+        end
+        # :nocov:
+      end
     end
   end
 end
